#!/bin/bash

echo "**********************************************"
echo "BOUSI-PROLOG source v3.6.1 installation script"
echo "**********************************************"

# Checks if root is going to install Bousi-Prolog and
# sets the default installation directory 
if [ $UID -eq 0 ]; then
	echo
	echo "You are logged in as root."
	echo -n "Do you want to install Bousi-Prolog for all users or for root only (A/R)? "
	REPEAT=1
	while [ $REPEAT -eq 1 ]; do
		read -n 1 -s OPTION
		if [ "$OPTION" = "R" ] || [ "$OPTION" = "r" ]; then
			# Default directory is root home
			echo $OPTION
			INSTALLDIR="$HOME/bousi-source"
			ALLUSERS=0
			REPEAT=0
		elif [ "$OPTION" = "A" ] || [ "$OPTION" = "a" ]; then
			# Default directory is public /usr/local
			echo $OPTION
			INSTALLDIR="/usr/local/bousi-source"
			ALLUSERS=1
			REPEAT=0
		fi
	done
else
	# Default directory is user home
	INSTALLDIR="$HOME/bousi-source"
	ALLUSERS=0
fi

# Asks for the installation directory
REPEAT=1
while [ $REPEAT -eq 1 ]; do
	echo
	if [ $ALLUSERS -eq 1 ]; then
		echo "Bousi-Prolog source will be installed for all users at $INSTALLDIR"
	else
		echo "Bousi-Prolog source will be installed at $INSTALLDIR"
	fi
	echo -n "Do you want to change the installation path (Y/N)? "
	REPEATB=1
	while [ $REPEATB -eq 1 ]; do
		read -n 1 -s OPTION
		if [ "$OPTION" = "Y" ] || [ "$OPTION" = "y" ]; then
			# Changes the installation path
			echo $OPTION
			echo -n "Enter the installation path: "
			read INSTALLDIR
			REPEATB=0
		elif [ "$OPTION" = "N" ] || [ "$OPTION" = "n" ]; then
			# Accepts the current installation path
			echo $OPTION
			REPEATB=0
			REPEAT=0
		fi
	done
done


# Copies source code files
echo
echo "Copying source code files..."
mkdir -p "$INSTALLDIR"
mkdir -p "$INSTALLDIR/source"
mkdir -p "$INSTALLDIR/wn"
mkdir -p "$INSTALLDIR/examples"
mkdir -p "$INSTALLDIR/extern"
mkdir -p "$INSTALLDIR/test"
mkdir -p "$INSTALLDIR/doc"
cp "files/source/"*.pl "$INSTALLDIR/source"
cp "files/wn/"*.pl "$INSTALLDIR/wn"
cp "files/examples/"*.bpl "$INSTALLDIR/examples"
cp "files/extern/"*.c "$INSTALLDIR/extern"
cp "files/extern/"*.h "$INSTALLDIR/extern"
cp "files/extern/"*.lex "$INSTALLDIR/extern"
cp "files/extern/Makefile" "$INSTALLDIR/extern"
cp "files/test/"*.pl "$INSTALLDIR/test"
cp "files/test/"*.bpl "$INSTALLDIR/test"
cp "files/test/shell_inputs" "$INSTALLDIR/test"
cp "files/doc/"*.pl "$INSTALLDIR/doc"
cp "files/doc/pldoc.sty" "$INSTALLDIR/doc"
cp "files/doc/Makefile" "$INSTALLDIR/doc"
cp "files/Makefile" "$INSTALLDIR"
cp "files/bplInfo.sh" "$INSTALLDIR"
cp "files/uninstaller.sh" "$INSTALLDIR"


# Sets file permissions if Bousi-Prolog has been installed for
# all users to avoid changing or deleting source code
if [ $ALLUSERS -eq 1 ]; then
	chmod u=rw,go=r "$INSTALLDIR/"*
	chmod u=rwx,go=r "$INSTALLDIR/"*.sh
	chmod u=rwx,go=rx "$INSTALLDIR/source"
	chmod u=rwx,go=rx "$INSTALLDIR/wn"
	chmod u=rwx,go=rx "$INSTALLDIR/examples"
	chmod u=rwx,go=rx "$INSTALLDIR/extern"
	chmod u=rwx,go=rx "$INSTALLDIR/test"
	chmod u=rwx,go=rx "$INSTALLDIR/doc"
	chmod u=rw,go=r "$INSTALLDIR/source/"*
	chmod u=rw,go=r "$INSTALLDIR/wn/"*
	chmod u=rw,go=r "$INSTALLDIR/examples/"*
	chmod u=rw,go=r "$INSTALLDIR/extern/"*
	chmod u=rw,go=r "$INSTALLDIR/test/"*
	chmod u=rw,go=r "$INSTALLDIR/doc/"*
fi

# If Bousi-Prolog is being installed under Cygwin, "cygpl.dll" library
# must be in the same directory as the main executable
CYGPLDLL=`find /lib/pl-*/bin/i*-cygwin/cygpl.dll 2> /dev/null`
if [ $CYGPLDLL ] && ! [ -f "$INSTALLDIR/cygpl.dll" ]; then
	ln $CYGPLDLL "$INSTALLDIR/cygpl.dll"
fi

# Makes additional changes to the system
if [ $ALLUSERS -eq 1 ]; then
	echo
	echo "Creating a script to launch Bousi-Prolog in /usr/local/bin..."
	# Creates a script to launch Bousi-Prolog in /usr/local/bin so any
	# user can start the program entering "bousi" in a shell
	echo "#!/bin/bash" > /usr/local/bin/bousi
	echo "\"$INSTALLDIR/bousi\"" >> /usr/local/bin/bousi
	chmod u=rwx,go=rx /usr/local/bin/bousi
else
	echo
	echo "Adding Bousi-Prolog installation directory to PATH..."
	# Retrieves the name of the script that is executed every time a new
	# shell is opened; we use ".bashrc" on Linux and ".profile" on Mac OS X
	if [[ $OSTYPE = linux* ]]; then
		INITSCRIPT=.bashrc
	else
		INITSCRIPT=.profile
	fi
	# Checks if Bousi-Prolog is installed and ".bplrc" is
	# already launched within ".bashrc" or ".profile"
	cat "$HOME/$INITSCRIPT" 2> /dev/null | grep -x ". \"\$HOME/.bplrc\"" > /dev/null
	if [ $? -eq 1 ]; then
		# Modifies ".bashrc" or ".profile" to call the Bousi-Prolog
		# initialization script (".bplrc") when launching a new shell
		LASTLINE=`cat "$HOME/$INITSCRIPT" 2> /dev/null | tail -n 1`
		if [ "$LASTLINE" != "" ]; then
			echo >> "$HOME/$INITSCRIPT"
		fi
		echo "# Bousi-Prolog initialization" >> "$HOME/$INITSCRIPT"
		echo ". \"\$HOME/.bplrc\"" >> "$HOME/$INITSCRIPT"
	fi
	# Creates the ".bplrc" file that adds Bousi-Prolog directory to PATH
	echo "# Adds Bousi-Prolog home directory to PATH" > "$HOME/.bplrc"
	echo "BPL_PATH=\"$INSTALLDIR\"" >> "$HOME/.bplrc"
	echo "export PATH=\"\$PATH:\$BPL_PATH\"" >> "$HOME/.bplrc"
	if [[ $OSTYPE != linux* ]]; then
		# On Mac OS X, the default installation directory of SWI-Prolog is
		# /opt/local, if you use Macports, or /usr/local/Cellar, if you use
        # Homebrew installation package.These are not in the PATH; that's why
		# we need to add the following lines to the ".bplrc" initialization script
		echo >> "$HOME/.bplrc"
		echo "# Adds SWI-Prolog directories to PATH" >> "$HOME/.bplrc"
        echo "export PATH=\"\$PATH:/opt/local/bin:/opt/local/sbin\"" >> "$HOME/.bplrc"
        echo "export MANPATH=\"\$MANPATH:/usr/local/Cellar/swi-prolog/7.6.4/share/man:/opt/local/man:/opt/local/share/man\"" >> "$HOME/.bplrc"
		echo "export INFOPATH=\"\$INFOPATH:/opt/local/share/info\"" >> "$HOME/.bplrc"
	fi
	# Updates environment variables for this shell
	export PATH="$PATH:$INSTALLDIR"
	if [[ $OSTYPE != linux* ]]; then
        export PATH="$PATH:/opt/local/bin:/opt/local/sbin"
        export MANPATH="$MANPATH:/usr/local/Cellar/swi-prolog/7.6.4/share/man:/opt/local/man:/opt/local/share/man"
		export INFOPATH="$INFOPATH:/opt/local/share/info"
	fi
fi

# Asks whether the source code should be compiled now
echo
echo -n "Do you want to compile Bousi-Prolog now (Y/N)? "
REPEAT=1
while [ $REPEAT -eq 1 ]; do
	read -n 1 -s OPTION
	if [ "$OPTION" = "Y" ] || [ "$OPTION" = "y" ]; then
		# Compiles the Bousi-Prolog system
		echo $OPTION
		echo
		echo "Building Bousi-Prolog system..."
		cd "$INSTALLDIR"
		make all
		COMPILED=1
		REPEAT=0
	elif [ "$OPTION" = "N" ] || [ "$OPTION" = "n" ]; then
		# Skips the compilation
		echo $OPTION
		COMPILED=0
		REPEAT=0
	fi
done

# Shows how to compile and/or start Bousi-Prolog
if [ $ALLUSERS -eq 1 ]; then
	echo
	if [ $COMPILED -eq 1 ]; then
		echo "Bousi-Prolog source code has been installed and compiled for all users."
		echo "Any user will be able to launch the Bousi-Prolog system entering \"bousi\""
		echo "in a shell."
	else
		echo "Bousi-Prolog source code has been installed for all users."
		echo "Go to the installation directory and enter \"make\" to compile the source"
		echo "code. Then, any user will be able to launch the Bousi-Prolog system entering"
		echo "\"bousi\" in a shell."
	fi
else
	echo
	if [ $COMPILED -eq 1 ]; then
		echo "Bousi-Prolog source code has been installed and compiled."
		echo "Enter \"bousi\" to launch the Bousi-Prolog system."
	else
		echo "Bousi-Prolog source code has been installed."
		echo "Go to the installation directory and enter \"make\" to compile the source"
		echo "code. Then, you will be able to launch the Bousi-Prolog system entering"
		echo "\"bousi\" in a shell."
	fi
fi

# Sets the installation directory as the new working directory
cd "$INSTALLDIR"

echo
echo "Press any key to exit the installation script."
read -n 1 -s

